@@ -7,7 +7,7 @@
 {
     OC_PythonUnicode* res;
 
-    res = [[OC_PythonUnicode alloc] initWithPythonObject:v];
+    res = [[self alloc] initWithPythonObject:v];
     return [res autorelease];
 }
 
@@ -43,6 +43,10 @@
     return YES;
 }
 
++ (BOOL)supportsSecureCoding {
+    return NO;
+}
+
 -(oneway void)release
 {
     /* There is small race condition when an object is almost deallocated
@@ -55,6 +59,16 @@
     /* FIXME: Should switch to __weak on OSX 10.7 or later, that should
      * fix this issue without a performance penalty.
      */
+    /*
+     * There is also a race condition on application shutdown between
+     * the call to Py_Finalize (shutting down the interpreter) and the
+     * cleanup performed by Cocoa, possible on other threads.
+     */
+    if (unlikely(!Py_IsInitialized())) {
+        [super release];
+        return;
+    }
+
     PyObjC_BEGIN_WITH_GIL
         [super release];
     PyObjC_END_WITH_GIL
@@ -62,6 +76,10 @@
 
 -(void)dealloc
 {
+    if (unlikely(!Py_IsInitialized())) {
+        [super dealloc];
+        return;
+    }
     PyObjC_BEGIN_WITH_GIL
         PyObjC_UnregisterObjCProxy(value, self);
         [realObject release];
@@ -254,8 +272,8 @@
     byteorder = 2;
     switch (encoding) {
     case NSASCIIStringEncoding: byteorder = 0; break;
-    case NSUTF8StringEncoding:  byteorder = -1; break;
-    case NSISOLatin1StringEncoding:  byteorder = 1; break;
+    case NSUTF8StringEncoding: byteorder = -1; break;
+    case NSISOLatin1StringEncoding: byteorder = 1; break;
     }
     if (byteorder != 2) {
         PyObjC_BEGIN_WITH_GIL
