@@ -12,7 +12,7 @@ call_NSCoder_encodeValueOfObjCType_at_(
     struct objc_super super;
     Py_ssize_t typestr_len;
 
-    if  (!PyArg_ParseTuple(arguments, Py_ARG_BYTES "#O", &typestr, &typestr_len, &value)) {
+    if (!PyArg_ParseTuple(arguments, Py_ARG_BYTES "#O", &typestr, &typestr_len, &value)) {
         return NULL;
     }
 
@@ -141,7 +141,7 @@ call_NSCoder_encodeArrayOfObjCType_count_at_(
     struct objc_super super;
     Py_ssize_t typestr_len;
 
-    if  (!PyArg_ParseTuple(arguments, Py_ARG_BYTES "#" Py_ARG_NSUInteger "O", &typestr, &typestr_len, &count, &value)) {
+    if (!PyArg_ParseTuple(arguments, Py_ARG_BYTES "#" Py_ARG_NSUInteger "O", &typestr, &typestr_len, &count, &value)) {
         return NULL;
     }
 
@@ -228,8 +228,8 @@ imp_NSCoder_encodeArrayOfObjCType_count_at_(
     PyObject* arglist = NULL;
     PyObject* v = NULL;
     PyObject* values = NULL;
-    Py_ssize_t    size;
-    NSUInteger     i;
+    Py_ssize_t size;
+    NSUInteger i;
     PyObject* pyself = NULL;
     int cookie = 0;
 
@@ -295,12 +295,12 @@ call_NSCoder_decodeValueOfObjCType_at_(
 {
     char* typestr;
     PyObject* value;
-    void*     buf;
-    Py_ssize_t    size, typestr_len;
+    void* buf;
+    Py_ssize_t size, typestr_len;
     struct objc_super super;
     PyObject* py_buf;
 
-    if  (!PyArg_ParseTuple(arguments, Py_ARG_BYTES "#O", &typestr, &typestr_len, &py_buf)) {
+    if (!PyArg_ParseTuple(arguments, Py_ARG_BYTES "#O", &typestr, &typestr_len, &py_buf)) {
         return NULL;
     }
 
@@ -409,21 +409,142 @@ error:
     return;
 }
 
+static PyObject*
+call_NSCoder_decodeValueOfObjCType_at_size_(
+    PyObject* method, PyObject* self, PyObject* arguments)
+{
+    char* typestr;
+    PyObject* value;
+    void* buf;
+    Py_ssize_t size, typestr_len;
+    struct objc_super super;
+    PyObject* py_buf;
+
+    if (!PyArg_ParseTuple(arguments, Py_ARG_BYTES "#On", &typestr, &typestr_len, &py_buf, &size)) {
+        return NULL;
+    }
+
+    if (py_buf != Py_None) {
+        PyErr_SetString(PyExc_ValueError, "buffer must be None");
+        return NULL;
+    }
+
+    buf = PyMem_Malloc(size);
+    if (buf == NULL) {
+        PyErr_NoMemory();
+        return NULL;
+    }
+
+    int isIMP = PyObjCIMP_Check(method);
+    PyObjC_DURING
+        if (isIMP) {
+            ((void(*)(id,SEL, char*,void*, NSUInteger))
+                (PyObjCIMP_GetIMP(method)))(
+                    PyObjCObject_GetObject(self),
+                    PyObjCIMP_GetSelector(method),
+                    typestr, buf, size);
+
+        } else {
+            objc_superSetReceiver(super, PyObjCObject_GetObject(self));
+            objc_superSetClass(super, PyObjCSelector_GetClass(method));
+
+            (void)objc_msgSendSuper(&super,
+                    PyObjCSelector_GetSelector(method),
+                    typestr, buf, size);
+        }
+
+    PyObjC_HANDLER
+        PyObjCErr_FromObjC(localException);
+
+    PyObjC_ENDHANDLER
+
+    if (PyErr_Occurred()) {
+        PyMem_Free(buf);
+        return NULL;
+    }
+
+    value = pythonify_c_value(typestr, buf);
+    PyMem_Free(buf);
+    if (value == NULL) {
+        return NULL;
+    }
+
+    return value;
+}
+
+static void
+imp_NSCoder_decodeValueOfObjCType_at_size_(
+    ffi_cif* cif __attribute__((__unused__)),
+    void* resp __attribute__((__unused__)),
+    void** args,
+    void* callable)
+{
+    id self = *(id*)args[0];
+    char* typestr = *(char**)args[2];
+    void* buf = *(void**)args[3];
+    NSUInteger size = *(NSUInteger*)args[4];
+
+    PyObject* result = NULL;
+    PyObject* arglist = NULL;
+    PyObject* v;
+    int err;
+    PyObject* pyself = NULL;
+    int cookie = 0;
+
+    PyGILState_STATE state = PyGILState_Ensure();
+
+    arglist = PyTuple_New(3);
+    if (arglist == NULL) goto error;
+
+    pyself = PyObjCObject_NewTransient(self, &cookie);
+    if (pyself == NULL) goto error;
+    PyTuple_SetItem(arglist, 0, pyself);
+    Py_INCREF(pyself);
+
+    v = PyBytes_FromString(typestr);
+    if (v == NULL) goto error;
+    PyTuple_SetItem(arglist, 1, v);
+
+    v = PyLong_FromLong(size);
+    if (v == NULL) goto error;
+    PyTuple_SetItem(arglist, 2, v);
+
+    result = PyObject_Call((PyObject*)callable, arglist, NULL);
+    Py_DECREF(arglist); arglist = NULL;
+    PyObjCObject_ReleaseTransient(pyself, cookie); pyself = NULL;
+    if (result == NULL) goto error;
+
+    err = depythonify_c_value(typestr, result, buf); // XXX
+    Py_DECREF(result);
+    if (err == -1) goto error;
+
+    PyGILState_Release(state);
+    return;
+
+error:
+    Py_XDECREF(arglist);
+    if (pyself) {
+        PyObjCObject_ReleaseTransient(pyself, cookie);
+    }
+    PyObjCErr_ToObjCWithGILState(&state);
+    return;
+}
+
 static PyObject*
 call_NSCoder_decodeArrayOfObjCType_count_at_(
     PyObject* method, PyObject* self, PyObject* arguments)
 {
     char* typestr;
-    NSUInteger   count;
-    NSUInteger   i;
+    NSUInteger count;
+    NSUInteger i;
     PyObject* result;
     PyObject* py_buf;
-    void*     buf;
-    Py_ssize_t    size;
+    void* buf;
+    Py_ssize_t size;
     struct objc_super super;
     Py_ssize_t typestr_len;
 
-    if  (!PyArg_ParseTuple(arguments,
+    if (!PyArg_ParseTuple(arguments,
                 Py_ARG_BYTES "#" Py_ARG_NSUInteger "O",
                 &typestr, &typestr_len, &count, &py_buf)) {
         return NULL;
@@ -480,7 +601,7 @@ call_NSCoder_decodeArrayOfObjCType_count_at_(
     }
 
     for (i = 0; i < count; i++) {
-        PyTuple_SetItem(result, i,  pythonify_c_value(typestr,
+        PyTuple_SetItem(result, i, pythonify_c_value(typestr,
                 ((char*)buf) + (size * i)));
         if (PyTuple_GetItem(result, i) == NULL) {
             Py_DECREF(result);
@@ -583,7 +704,7 @@ call_NSCoder_encodeBytes_length_(
 
     struct objc_super super;
 
-    if  (!PyArg_ParseTuple(arguments, Py_ARG_BYTES"#n", &bytes, &size, &length)) {
+    if (!PyArg_ParseTuple(arguments, Py_ARG_BYTES"#n", &bytes, &size, &length)) {
         return NULL;
     }
 
@@ -688,7 +809,7 @@ call_NSCoder_decodeBytesWithReturnedLength_(
     PyObject* py_buf;
     struct objc_super super;
 
-    if  (!PyArg_ParseTuple(arguments, "O", &py_buf)) {
+    if (!PyArg_ParseTuple(arguments, "O", &py_buf)) {
         return NULL;
     }
     if (py_buf != Py_None) {
@@ -831,7 +952,7 @@ imp_NSCoder_decodeBytesWithReturnedLength_(
     /* Should return an autoreleased buffer, do this by createing an
      * NSData that will release the buffer
      */
-    *pretval =  (const void*)[[[[NSData alloc] initWithBytes:*pretval length:len]
+    *pretval = (const void*)[[[[NSData alloc] initWithBytes:*pretval length:len]
           autorelease] bytes];
 
     Py_DECREF(result);
@@ -859,7 +980,7 @@ call_NSCoder_decodeBytesForKey_returnedLength_(
     id key;
     struct objc_super super;
 
-    if  (!PyArg_ParseTuple(arguments, "O&O", PyObjCObject_Convert, &key, &py_buf)) {
+    if (!PyArg_ParseTuple(arguments, "O&O", PyObjCObject_Convert, &key, &py_buf)) {
         return NULL;
     }
 
@@ -1008,7 +1129,7 @@ imp_NSCoder_decodeBytesForKey_returnedLength_(
     /* Should return an autoreleased buffer, do this by createing an
      * NSData that will release the buffer
      */
-    *pretval =  (const void*)[[[[NSData alloc] initWithBytes:*pretval length:len]
+    *pretval = (const void*)[[[[NSData alloc] initWithBytes:*pretval length:len]
           autorelease] bytes];
 
     Py_DECREF(result);
@@ -1034,7 +1155,7 @@ call_NSCoder_encodeBytes_length_forKey_(
     id key;
     struct objc_super super;
 
-    if  (!PyArg_ParseTuple(arguments, Py_ARG_BYTES "#O&", &bytes, &size,
+    if (!PyArg_ParseTuple(arguments, Py_ARG_BYTES "#O&", &bytes, &size,
             PyObjCObject_Convert, &key)) {
         return NULL;
     }
@@ -1167,6 +1288,14 @@ int PyObjC_setup_nscoder(void)
         return -1;
     }
 
+    if (PyObjC_RegisterMethodMapping(
+            classNSCoder,
+            @selector(decodeValueOfObjCType:at:size:),
+            call_NSCoder_decodeValueOfObjCType_at_size_,
+            imp_NSCoder_decodeValueOfObjCType_at_size_) < 0) {
+        return -1;
+    }
+
     if (PyObjC_RegisterMethodMapping(
             classNSCoder,
             @selector(encodeBytes:length:),
