@@ -14,6 +14,8 @@ PyObject* PyObjCExc_ObjCRevivalWarning = NULL;
 PyObject* PyObjCExc_LockError = NULL;
 PyObject* PyObjCExc_BadPrototypeError = NULL;
 PyObject* PyObjCExc_UnknownPointerError = NULL;
+PyObject* PyObjCExc_DeprecationWarning = NULL;
+PyObject* PyObjCExc_ObjCPointerWarning = NULL;
 
 int
 PyObjCUtil_Init(PyObject* module)
@@ -32,6 +34,8 @@ PyObjCUtil_Init(PyObject* module)
     NEW_EXC(PyObjCExc_LockError, "LockError", PyObjCExc_Error);
     NEW_EXC(PyObjCExc_BadPrototypeError, "BadPrototypeError", PyObjCExc_Error);
     NEW_EXC(PyObjCExc_UnknownPointerError, "UnknownPointerError", PyObjCExc_Error);
+    NEW_EXC(PyObjCExc_DeprecationWarning, "ApiDeprecationWarning", PyExc_DeprecationWarning);
+    NEW_EXC(PyObjCExc_ObjCPointerWarning, "ObjCPointerWarning", PyExc_Warning);
 
 #undef NEW_EXC
     return 0;
@@ -43,13 +47,13 @@ ObjCErr_PyExcForName(const char* value)
     if (strcmp(value, "NSRangeException") == 0) {
         return PyExc_IndexError;
 
-    }  else if (strcmp(value, "NSInvalidArgumentException") == 0) {
+    } else if (strcmp(value, "NSInvalidArgumentException") == 0) {
         return PyExc_ValueError;
 
-    }  else if (strcmp(value, "NSMallocException") == 0) {
+    } else if (strcmp(value, "NSMallocException") == 0) {
         return PyExc_MemoryError;
 
-    }  else if (strcmp(value, "NSUnknownKeyException") == 0) {
+    } else if (strcmp(value, "NSUnknownKeyException") == 0) {
         return PyExc_KeyError;
     }
 
@@ -102,8 +106,8 @@ PyObjCErr_FromObjC(NSException* localException)
                 val = [userInfo objectForKey:@"__pyobjc_exc_type__"];
                 if (val) {
                     exc_type = [val pyObject];
-                    exc_value = [[userInfo objectForKey:@"__pyobjc_exc_value__"]  pyObject];
-                    exc_traceback = [[userInfo objectForKey:@"__pyobjc_exc_traceback__"]  pyObject];
+                    exc_value = [[userInfo objectForKey:@"__pyobjc_exc_value__"] pyObject];
+                    exc_traceback = [[userInfo objectForKey:@"__pyobjc_exc_traceback__"] pyObject];
 
                     /* -pyObject returns a borrowed reference and
                      * PyErr_Restore steals one from us.
@@ -140,7 +144,7 @@ PyObjCErr_FromObjC(NSException* localException)
             PyDict_SetItemString(dict, "name", c_localException_name);
             Py_DECREF(c_localException_name);
 
-            PyDict_SetItemString(dict, "reason",  c_localException_reason);
+            PyDict_SetItemString(dict, "reason", c_localException_reason);
             Py_DECREF(c_localException_reason);
             if (userInfo) {
                 v = PyObjCObject_New(userInfo, PyObjCObject_kDEFAULT, YES);
@@ -375,7 +379,7 @@ NSMapTableKeyCallBacks PyObjCUtil_ObjCIdentityKeyCallBacks = {
 NSMapTableValueCallBacks PyObjCUtil_ObjCValueCallBacks = {
     &nsmaptable_objc_retain,
     &nsmaptable_objc_release,
-    NULL  // generic description
+    NULL // generic description
 };
 
 
@@ -462,9 +466,9 @@ static int
 buffer_get(BOOL writable, PyObject* obj, void** bufptr, Py_ssize_t* sizeptr)
 {
     if (writable) {
-        return PyObject_AsWriteBuffer(obj, bufptr,  sizeptr);
+        return PyObject_AsWriteBuffer(obj, bufptr, sizeptr);
     } else {
-        return PyObject_AsReadBuffer(obj, (const void**)bufptr,  sizeptr);
+        return PyObject_AsReadBuffer(obj, (const void**)bufptr, sizeptr);
     }
 }
 
@@ -668,12 +672,12 @@ code_compatible(char array_code, char type_code)
  */
 int
 PyObjC_PythonToCArray(
-    BOOL        writable, BOOL exactSize,
+    BOOL writable, BOOL exactSize,
     const char* elementType,
-    PyObject*   pythonList,
+    PyObject*  pythonList,
     void** array,
-    Py_ssize_t*   size,
-    PyObject**    bufobj)
+    Py_ssize_t*  size,
+    PyObject** bufobj)
 {
     Py_ssize_t eltsize = PyObjCRT_SizeOfType(elementType);
     Py_ssize_t i;
@@ -773,17 +777,6 @@ PyObjC_PythonToCArray(
     }
 
     if (*elementType == _C_UNICHAR && PyUnicode_Check(pythonList)) {
-        Py_ssize_t bufsize = PyUnicode_GetSize(pythonList);
-
-        if (*size == -1) {
-            *size = bufsize;
-
-        } else if ((exactSize && *size != bufsize) || (!exactSize && *size > bufsize)) {
-            PyErr_Format(PyExc_ValueError,
-                "Requesting unicode buffer of %"PY_FORMAT_SIZE_T"d, have unicode buffer "
-                "of %"PY_FORMAT_SIZE_T"d", *size, bufsize);
-            return -1;
-        }
 
 #if PY_VERSION_HEX >= 0x03030000
         *bufobj = _PyUnicode_EncodeUTF16(
@@ -799,6 +792,21 @@ PyObjC_PythonToCArray(
             return -1;
         }
 
+        Py_ssize_t bufsize = PyBytes_Size(*bufobj) / 2;
+
+        if (*size == -1) {
+            *size = bufsize;
+
+        } else if ((exactSize && *size != bufsize) || (!exactSize && *size > bufsize)) {
+            /* NOTE: The size check is performed after the conversion to UTF16 to avoid
+             * problems with characters outside of the BMP.
+             */
+            PyErr_Format(PyExc_ValueError,
+                "Requesting unicode buffer of %"PY_FORMAT_SIZE_T"d, have unicode buffer "
+                "of %"PY_FORMAT_SIZE_T"d", *size, bufsize);
+            return -1;
+        }
+
         /* XXX: Update API protocol to make the extra copy not necessary
          * Cannot use the code at the end because 'buffer' is assumed to be
          * the value to return to the python caller.
@@ -811,7 +819,7 @@ PyObjC_PythonToCArray(
 
         /* *array = PyBytes_AsString(*bufobj); return SHOULD_IGNORE*/
 
-#else    /* Python before 3.3 */
+#else /* Python before 3.3 */
         if (writable) {
             *array = PyMem_Malloc(*size * sizeof(UniChar));
             memcpy(*array, PyUnicode_AsUnicode(pythonList), *size * sizeof(UniChar));
@@ -823,7 +831,7 @@ PyObjC_PythonToCArray(
             Py_INCREF(pythonList);
             return SHOULD_IGNORE;
         }
-#endif  /* Python before 3.3 */
+#endif /* Python before 3.3 */
 
 #if PY_MAJOR_VERSION == 2
     } else if (*elementType == _C_UNICHAR && PyString_Check(pythonList)) {
@@ -1098,6 +1106,7 @@ PyObjC_IsPythonKeyword(const char* word)
     static const char* keywords[] = {
         "class",
         "raise",
+        "from",
         NULL
     };
     const char** cur;
@@ -1281,9 +1290,9 @@ int PyObjC_is_ascii_string(PyObject* unicode_string, const char* ascii_string)
 
 int PyObjC_is_ascii_prefix(PyObject* unicode_string, const char* ascii_string, size_t n)
 {
-    size_t uni_sz = PyUnicode_GetSize(unicode_string);
 
 #ifdef PyObjC_FAST_UNICODE_ASCII
+    size_t uni_sz = PyUnicode_GetLength(unicode_string);
 
     if (uni_sz < n) {
         return 0;
@@ -1297,6 +1306,7 @@ int PyObjC_is_ascii_prefix(PyObject* unicode_string, const char* ascii_string, s
 
 #else /* !PyObjC_FAST_UNICODE_ASCII */
 
+    size_t uni_sz = PyUnicode_GetSize(unicode_string);
     size_t i;
     Py_UNICODE* code_points = PyUnicode_AsUnicode(unicode_string);
 
