@@ -10,6 +10,7 @@
 #define PyObjCSelector_kHIDDEN                0x000002
 #define PyObjCSelector_kREQUIRED              0x000004
 #define PyObjCSelector_kRETURNS_UNINITIALIZED 0x000010
+#define PyObjCSelector_kNATIVE		      0x000020
 
 
 
@@ -25,13 +26,14 @@ typedef PyObject* (*PyObjC_CallFunc)(
 
 typedef struct {
     PyObject_HEAD
-    const char*  sel_python_signature;
-    const char*  sel_native_signature;
-    SEL          sel_selector;
-    PyObject*    sel_self;
-    Class        sel_class;
-    int          sel_flags;
+    const char* sel_python_signature;
+    const char* sel_native_signature;
+    SEL sel_selector;
+    PyObject* sel_self;
+    Class sel_class;
+    int sel_flags;
     PyObjCMethodSignature* sel_methinfo;
+    Py_ssize_t	sel_mappingcount;
 } PyObjCSelector;
 
 typedef struct {
@@ -41,9 +43,9 @@ typedef struct {
 
 typedef struct {
     PyObjCSelector base;
-    PyObject*    callable;
-    Py_ssize_t    argcount;
-    Py_ssize_t    numoutput;
+    PyObject* callable;
+    Py_ssize_t argcount;
+    Py_ssize_t numoutput;
 } PyObjCPythonSelector;
 
 extern PyTypeObject PyObjCSelector_Type;
