@@ -17,6 +17,7 @@ if sys.version_info[0] == 3:
     long = int
 
 OC_PythonNumber = objc.lookUpClass("OC_PythonNumber")
+OC_BuiltinPythonNumber = objc.lookUpClass("OC_BuiltinPythonNumber")
 try:
     NSCFNumber = objc.lookUpClass("__NSCFNumber")
 except objc.error:
@@ -52,7 +53,7 @@ class TestNSNumber (TestCase):
         v = NSNumber.numberWithUnsignedLongLong_(2 ** 63 + 5000)
         self.assertIsInstance(v, long)
 
-        if os_release() <= '10.5' and os_release() != '10.10':
+        if os_level_key(os_release()) < os_level_key('10.5'):
             self.assertEqual(v.description(), str(-2**63+5000))
         else:
             self.assertEqual(v.description(), str(2**63+5000))
@@ -66,10 +67,12 @@ class TestNSNumber (TestCase):
 
         n = objc.lookUpClass('NSObject').alloc().init()
 
-        with filterWarnings("error", RuntimeWarning):
-            self.assertRaises(RuntimeWarning, numberWrapper, n)
+        with warnings.catch_warnings(record=True) as w:
+            numberWrapper(n)
+        self.assertEqual(len(w), 1)
+        self.assertEqual(w[0].category, RuntimeWarning)
 
-        with filterWarnings("ignore", RuntimeWarning):
+        with warnings.catch_warnings():
             self.assertIs(numberWrapper(n), n)
 
 
@@ -97,7 +100,7 @@ class TestNSNumber (TestCase):
         data = pickle.dumps(v)
 
         w = pickle.loads(data)
-        if os_release() <= '10.5' and os_release() != '10.10':
+        if os_level_key(os_release()) < os_level_key('10.5'):
             self.assertEqual(w, {
                 'long': -2**63 + 5000,
                 'int': 42,
@@ -191,7 +194,6 @@ class TestNSNumber (TestCase):
     def testDoubleConversions(self):
         v = NSNumber.numberWithDouble_(75.5)
         self.assertEqual(v.stringValue(), '75.5')
-
         self.assertEqual(OC_TestNumber.numberAsBOOL_(v), 1)
         self.assertEqual(OC_TestNumber.numberAsChar_(v), 75)
         self.assertEqual(OC_TestNumber.numberAsShort_(v), 75)
@@ -293,7 +295,7 @@ class TestPyNumber (TestCase):
     def testClasses(self):
         # Ensure that python numbers are proxied using the right proxy type
         for v in (0, 1, 2**32+1, 2**64+1, 42.5):
-            self.assertIs(OC_TestNumber.numberClass_(v), OC_PythonNumber)
+            self.assertIs(OC_TestNumber.numberClass_(v), OC_BuiltinPythonNumber)
 
         # The booleans True and False must be proxied as the corresponding
         # NSNumber constants, otherwise lowlevel Cocoa/CoreFoundation code
@@ -361,6 +363,25 @@ class TestPyNumber (TestCase):
         self.assertEqual(OC_TestNumber.numberAsUnsignedChar_(v), 39)
         self.assertEqual(OC_TestNumber.numberAsUnsignedShort_(v), 40487)
 
+
+        # Python long
+        v = long(42)
+
+        self.assertEqual(OC_TestNumber.numberAsBOOL_(v), 1)
+        self.assertEqual(OC_TestNumber.numberAsChar_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsShort_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsInt_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsLong_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsLongLong_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsUnsignedChar_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsUnsignedShort_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsUnsignedInt_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsUnsignedLong_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsUnsignedLongLong_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsUnsignedLongLong_(v), 42)
+        self.assertEqual(OC_TestNumber.numberAsFloat_(v), 42.0)
+        self.assertEqual(OC_TestNumber.numberAsDouble_(v), 42.0)
+
     def testPythonLongConversions(self):
         if sys.version_info[0] == 2:
             v = long(42)
@@ -472,10 +493,10 @@ class TestPyNumber (TestCase):
         else:
             self.assertEqual(OC_TestNumber.numberAsUnsignedLong_(v), 18446744073709551489)
 
-        if sys.byteorder == 'big':
-            self.assertEqual(OC_TestNumber.numberAsUnsignedLongLong_(v), 4294967169)
-        else:
-            self.assertEqual(OC_TestNumber.numberAsUnsignedLongLong_(v), 18446744073709551489)
+        #if sys.byteorder == 'big':
+        #    self.assertEqual(OC_TestNumber.numberAsUnsignedLongLong_(v), 4294967169)
+        #else:
+        self.assertEqual(OC_TestNumber.numberAsUnsignedLongLong_(v), 18446744073709551489)
 
         self.assertEqual(OC_TestNumber.numberAsDouble_(v), -127.6)
 
