@@ -6,16 +6,6 @@ import ctypes
 
 from PyObjCTools import TestSupport
 
-try:
-    long
-except NameError:
-    long = int
-
-try:
-    unicode
-except NameError:
-    unicode = str
-
 try:
     from StringIO import StringIO
 
@@ -108,7 +98,7 @@ class TestTestSupport (TestCase):
         self.assertEqual(os_release(), '10.10')
         TestSupport._os_release = None
 
-        self.assertEqual(TestSupport.os_release(), '.'.join(platform.mac_ver()[0].split('.')[:2]))
+        self.assertEqual(TestSupport.os_release(), '.'.join(platform.mac_ver()[0].split('.')))
 
     def test_fourcc(self):
         import struct
@@ -1248,31 +1238,6 @@ class TestTestSupport (TestCase):
         else:
             self.fail("unexpected test pass")
 
-    def test_filterWarnings(self):
-        class Warn1 (RuntimeWarning): pass
-        class Warn2 (RuntimeWarning): pass
-        class Warn3 (RuntimeWarning): pass
-
-        orig_stderr = sys.stderr
-        sys.stderr = StringIO()
-        try:
-            with filterWarnings('error', Warn1):
-                self.assertRaises(Warn1, warnings.warn, 'helper', category=Warn1)
-
-            self.assertEqual(sys.stderr.getvalue(), '')
-
-            with filterWarnings('ignore', Warn2):
-                warnings.warn('helper', category=Warn2)
-
-            self.assertEqual(sys.stderr.getvalue(), '')
-
-            warnings.warn('helper', category=Warn3)
-
-            self.assertNotEqual(sys.stderr.getvalue(), '')
-        finally:
-            sys.stderr = orig_stderr
-
-
     def test_running(self):
         orig_use    = TestSupport._usepool
         orig_class = TestSupport._poolclass
